package com.emato.ccnet.aop;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * Created by Anyx on 2015/5/13.
 * 日志AOP
 */

@Aspect
@Component
public class AspectWebLog {
    private static final Logger logger = LoggerFactory.getLogger(AspectWebLog.class);


    @Autowired
    ObjectMapper objectMapper;

    @Pointcut("execution(public * com.emato.ccnet.controller..*.*(..))")
    public void controllerPointcut(){

    }

    /**
     * 前置通知
     * 执行方法之前执行前置通知方法
     * @param jp:JoinPoint
     * @return
     */
    @Before(value="controllerPointcut()")
    public void beforeAdvice(JoinPoint jp) {

        String target = jp.getTarget().getClass().getName();
        String mothod = jp.getSignature().getName();
        try {
            Object[] args = jp.getArgs();

            String str = null;

            if (args.length > 0) {
                if (args[0] instanceof Map) {
                    Map<String, String> map = (Map<String, String>) args[0];
                    str = objectMapper.writeValueAsString(map);
                }else{
                    str = objectMapper.writeValueAsString(args[0]);
                }
                logger.info("【" + target + "#" + mothod + "】发起请求, 请求参数: {}", str);
            }

        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }

    }

    /**
     * 返回通知
     * 方法执行成功后，调用返回通知，如果方法在运行过程中抛出异常，则不会调用
     * @param jp:JoinPoint
     * @param retValue:String 主体方法传递到通知方法的返回值
     * @return
     */
    @AfterReturning(value="controllerPointcut()", returning = "retValue")
    public void afterReturningAdvice(JoinPoint jp, Object retValue) {

        String target = jp.getTarget().getClass().getName();
        String mothod = jp.getSignature().getName();

        if (retValue == null) {
            logger.info("【" + target + "#" + mothod + "】执行结束, 返回数据: {}");
            return;
        }
        try {
            String str = objectMapper.writeValueAsString(retValue);
            logger.info("【" + target + "#" + mothod + "】执行结束, 返回数据:{}", str);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
    }

    /**
     * 异常通知
     * 方法在运行过程中，如果抛出异常，则执行异常通知
     * @param jp:JoinPoint
     * @return
     */
    @AfterThrowing(value="controllerPointcut()", throwing = "ex")
    public void afterThrowingAdvice(JoinPoint jp, Exception ex) {
        String target = jp.getTarget().getClass().getName();
        String mothod = jp.getSignature().getName();
        logger.info("【" + target + "#" + mothod + "】发生异常结束: {}", ex);
    }

    /**
     * 后置通知
     * 执行方法之后执行后置通知方法
     * @param jp:JoinPoint
     * @return
     */
    @After(value="controllerPointcut()")
    public void afterAdvice(JoinPoint jp) {
        try {
            String target = jp.getTarget().getClass().getName();
            String mothod = jp.getSignature().getName();

            Object[] args = jp.getArgs();

            String str;

            if (args.length > 0) {
                if (args[0] instanceof Map) {
                    Map<String, String> map = (Map<String, String>) args[0];
                    str = objectMapper.writeValueAsString(map);
                }else{
                    str = objectMapper.writeValueAsString(args[0]);
                }
                logger.info("【" + target + "#" + mothod + "】执行结束, 返回数据: {}", str);
            }

        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }

    }

    /**
     * 环绕通知
     * 主体方法返回后将执行的通知方法
     * @param jp:ProceedingJoinPoint
     * @return
     */
    @Around("controllerPointcut()")
    public Object aroundAdvice(ProceedingJoinPoint jp) throws Throwable {
        String target = jp.getTarget().getClass().getName();
        String mothod = jp.getSignature().getName();

        long bftime = System.currentTimeMillis();
        Object retVal = jp.proceed();
        double time =(double)(System.currentTimeMillis() - bftime) / 1000;
        logger.debug("【" + target + "#" + mothod + "】执行时间: {}秒", time);
        return retVal;
    }

}
